@@ -5,9 +5,8 @@ import logging
 from collections import defaultdict
 from contextlib import contextmanager
 from enum import Enum
-from filelock import FileLock
-from packaging import version
-from typing import Dict, Tuple, Iterator, NamedTuple, Optional, List
+from cachetools.func import ttl_cache
+from typing import Dict, Tuple, Iterator, NamedTuple, Optional, List, Set
 from weakref import WeakValueDictionary
 
 from aim.ext.sshfs.utils import mount_remote_repo, unmount_remote_repo
@@ -21,8 +20,11 @@ from aim.sdk.run import Run
 from aim.sdk.utils import search_aim_repo, clean_repo_path
 from aim.sdk.sequence_collection import QuerySequenceCollection, QueryRunSequenceCollection
 from aim.sdk.sequence import Sequence
+from aim.sdk.types import QueryReportMode
 from aim.sdk.data_version import DATA_VERSION
+from aim.sdk.remote_repo_proxy import RemoteRepoProxy
 
+from aim.storage.locking import AutoFileLock
 from aim.storage.container import Container
 from aim.storage.rockscontainer import RocksContainer
 from aim.storage.union import RocksUnionContainer
@@ -65,10 +67,13 @@ class RepoAutoClean(AutoClean):
         """
         super().__init__(instance)
         self.root_path = instance.root_path
+        self._client = instance._client
         self._mount_root = instance._mount_root
 
     def _close(self) -> None:
         """Close the `Repo` and unmount the remote repository."""
+        if self._client:
+            self._client._heartbeat_sender.stop()
         if self._mount_root:
             logger.debug(f'Unmounting remote repository at {self._mount_root}')
             unmount_remote_repo(self.root_path, self._mount_root)
@@ -92,7 +97,7 @@ class Repo:
 
     tracking_queue = _get_tracking_queue()
 
-    def __init__(self, path: str, *, read_only: bool = None, init: bool = False):
+    def __init__(self, path: str, *, read_only: Optional[bool] = None, init: Optional[bool] = False):
         if read_only is not None:
             raise NotImplementedError
 
@@ -106,14 +111,13 @@ class Repo:
             remote_path = path.replace('aim://', '')
             self._client = Client(remote_path)
             self.root_path = remote_path
+            self._check_remote_version_compatibility()
         else:
             self.root_path = path
         self.path = os.path.join(self.root_path, get_aim_repo_name())
 
         if init:
             os.makedirs(self.path, exist_ok=True)
-            with open(os.path.join(self.path, 'VERSION'), 'w') as version_fh:
-                version_fh.write(DATA_VERSION + '\n')
         if not self.is_remote_repo and not os.path.exists(self.path):
             if self._mount_root:
                 unmount_remote_repo(self.root_path, self._mount_root)
@@ -129,12 +133,15 @@ class Repo:
 
         if not self.is_remote_repo:
             self._lock_path = os.path.join(self.path, '.repo_lock')
-            self._lock = FileLock(self._lock_path, timeout=10)
+            self._lock = AutoFileLock(self._lock_path, timeout=10)
 
             with self.lock():
+                status = self.check_repo_status(self.root_path)
                 self.structured_db = DB.from_path(self.path)
-                if init:
+                if init or status == RepoStatus.PATCH_REQUIRED:
                     self.structured_db.run_upgrades()
+                    with open(os.path.join(self.path, 'VERSION'), 'w') as version_fh:
+                        version_fh.write('.'.join(map(str, DATA_VERSION)) + '\n')
 
         self._resources = RepoAutoClean(self)
 
@@ -236,22 +243,61 @@ class Repo:
             return RepoStatus.UPDATED
         if not cls.exists(path):
             return RepoStatus.MISSING
-        repo_version = version.parse(cls.get_version(path))
-        current_version = version.parse(DATA_VERSION)
-        if repo_version.major < current_version.major:
-            return RepoStatus.UPDATE_REQUIRED
-        if repo_version.minor < current_version.minor:
-            return RepoStatus.PATCH_REQUIRED
+        repo_version = cls.get_version(path)
+        current_version = DATA_VERSION
+        if repo_version < current_version:
+            if repo_version[0] < current_version[0]:
+                return RepoStatus.UPDATE_REQUIRED
+            else:
+                return RepoStatus.PATCH_REQUIRED
         return RepoStatus.UPDATED
 
+    def _check_remote_version_compatibility(self):
+        assert self.is_remote_repo
+        from aim.__version__ import __version__ as client_version
+        import grpc
+
+        error_message_template = 'The Aim Remote tracking server version ({}) '\
+                                 'is not compatible with the Aim client version ({}).'\
+                                 'Please upgrade either the Aim Client or the Aim Remote.'
+
+        warning_message_template = 'The Aim Remote tracking server version ({}) ' \
+                                   'and the Aim client version ({}) do not match.' \
+                                   'Consider upgrading either the client or remote tracking server.'
+
+        try:
+            remote_version = self._client.get_version()
+        except grpc.RpcError as e:
+            if e.code() == grpc.StatusCode.UNIMPLEMENTED:
+                remote_version = '<3.12.0'
+            else:
+                raise
+
+        # server doesn't yet have the `get_version()` method implemented
+        if remote_version == '<3.12.0':
+            RuntimeError(error_message_template.format(remote_version, client_version))
+
+        # compare versions
+        if client_version == remote_version:
+            return
+
+        # if the server has a newer version always force to upgrade the client
+        if client_version < remote_version:
+            raise RuntimeError(error_message_template.format(remote_version, client_version))
+
+        # for other mismatching versions throw a warning for now
+        logger.warning(warning_message_template.format(remote_version, client_version))
+        # further incompatibility list will be added manually
+
     @classmethod
     def get_version(cls, path: str):
         path = clean_repo_path(path)
         version_file_path = os.path.join(path, get_aim_repo_name(), 'VERSION')
         if os.path.exists(version_file_path):
             with open(version_file_path, 'r') as version_fh:
-                return version_fh.read()
-        return '0.0'  # old Aim repos
+                version_tp = version_fh.read().strip().split('.')
+                return tuple(map(int, version_tp))
+        return 0, 0  # old Aim repos
 
     @classmethod
     def is_remote_path(cls, path: str):
@@ -384,6 +430,50 @@ class Repo:
         else:
             raise StopIteration
 
+    def run_exists(self, run_hash: str) -> bool:
+        return run_hash in self._all_run_hashes()
+
+    @ttl_cache(ttl=0.5)
+    def _all_run_hashes(self) -> Set[str]:
+        if self.is_remote_repo:
+            remote_repo = RemoteRepoProxy(self._client)
+            return set(remote_repo.list_all_runs())
+        else:
+            chunks_dir = os.path.join(self.path, 'meta', 'chunks')
+            if os.path.exists(chunks_dir):
+                return set(os.listdir(chunks_dir))
+            else:
+                return set()
+
+    def list_all_runs(self) -> List[str]:
+        return list(self._all_run_hashes())
+
+    def _active_run_hashes(self) -> Set[str]:
+        if self.is_remote_repo:
+            remote_repo = RemoteRepoProxy(self._client)
+            return set(remote_repo.list_active_runs())
+        else:
+            chunks_dir = os.path.join(self.path, 'meta', 'progress')
+            if os.path.exists(chunks_dir):
+                return set(os.listdir(chunks_dir))
+            else:
+                return set()
+
+    def list_active_runs(self) -> List[str]:
+        return list(self._active_run_hashes())
+
+    def total_runs_count(self) -> int:
+        db = self.structured_db
+        if db:
+            cache = db.caches.get('runs_cache')
+        else:
+            cache = None
+        if cache:
+            return len(cache.keys())
+        else:
+            self.meta_tree.preload()
+            return len(list(self.meta_tree.subtree('chunks').keys()))
+
     def get_run(self, run_hash: str) -> Optional['Run']:
         """Get run if exists.
 
@@ -398,7 +488,11 @@ class Repo:
         else:
             return Run(run_hash, repo=self, read_only=True)
 
-    def query_runs(self, query: str = '', paginated: bool = False, offset: str = None) -> QueryRunSequenceCollection:
+    def query_runs(self,
+                   query: str = '',
+                   paginated: bool = False,
+                   offset: str = None,
+                   report_mode: QueryReportMode = QueryReportMode.PROGRESS_BAR) -> QueryRunSequenceCollection:
         """Get runs satisfying query expression.
 
         Args:
@@ -406,11 +500,13 @@ class Repo:
                 If not specified, query results will include all runs.
              paginated (:obj:`bool`, optional): query results pagination flag. False if not specified.
              offset (:obj:`str`, optional): `hash` of Run to skip to.
+             report_mode(:obj:`QueryReportMode`, optional): indicates report mode
+                (0: DISABLED, 1: PROGRESS BAR, 2: PROGRESS TUPLE). QueryReportMode.PROGRESS_BAR if not specified.
         Returns:
             :obj:`SequenceCollection`: Iterable for runs/metrics matching query expression.
         """
         self._prepare_runs_cache()
-        return QueryRunSequenceCollection(self, Sequence, query, paginated, offset)
+        return QueryRunSequenceCollection(self, Sequence, query, paginated, offset, report_mode)
 
     def delete_run(self, run_hash: str) -> bool:
         """Delete Run data from aim repository
@@ -505,77 +601,101 @@ class Repo:
         else:
             return True, []
 
-    def query_metrics(self, query: str = '') -> QuerySequenceCollection:
+    def query_metrics(self,
+                      query: str = '',
+                      report_mode: QueryReportMode = QueryReportMode.PROGRESS_BAR) -> QuerySequenceCollection:
         """Get metrics satisfying query expression.
 
         Args:
              query (str): query expression.
+             report_mode(:obj:`QueryReportMode`, optional): indicates report mode
+                (0: DISABLED, 1: PROGRESS BAR, 2: PROGRESS TUPLE). QueryReportMode.PROGRESS_BAR if not specified.
         Returns:
-            :obj:`MetricCollection`: Iterable for metrics matching query expression.
+            :obj:`SequenceCollection`: Iterable for metrics matching query expression.
         """
         self._prepare_runs_cache()
         from aim.sdk.sequences.metric import Metric
-        return QuerySequenceCollection(repo=self, seq_cls=Metric, query=query)
+        return QuerySequenceCollection(repo=self, seq_cls=Metric, query=query, report_mode=report_mode)
 
-    def query_images(self, query: str = '') -> QuerySequenceCollection:
+    def query_images(self,
+                     query: str = '',
+                     report_mode: QueryReportMode = QueryReportMode.PROGRESS_BAR) -> QuerySequenceCollection:
         """Get image collections satisfying query expression.
 
         Args:
              query (str): query expression.
+             report_mode(:obj:`QueryReportMode`, optional): indicates report mode
+                (0: DISABLED, 1: PROGRESS BAR, 2: PROGRESS TUPLE). QueryReportMode.PROGRESS_BAR if not specified.
         Returns:
             :obj:`SequenceCollection`: Iterable for image sequences matching query expression.
         """
         self._prepare_runs_cache()
         from aim.sdk.sequences.image_sequence import Images
-        return QuerySequenceCollection(repo=self, seq_cls=Images, query=query)
+        return QuerySequenceCollection(repo=self, seq_cls=Images, query=query, report_mode=report_mode)
 
-    def query_audios(self, query: str = '') -> QuerySequenceCollection:
+    def query_audios(self,
+                     query: str = '',
+                     report_mode: QueryReportMode = QueryReportMode.PROGRESS_BAR) -> QuerySequenceCollection:
         """Get audio collections satisfying query expression.
 
         Args:
              query (str): query expression.
+             report_mode(:obj:`QueryReportMode`, optional): indicates report mode
+                (0: DISABLED, 1: PROGRESS BAR, 2: PROGRESS TUPLE). QueryReportMode.PROGRESS_BAR if not specified.
         Returns:
             :obj:`SequenceCollection`: Iterable for audio sequences matching query expression.
         """
         self._prepare_runs_cache()
         from aim.sdk.sequences.audio_sequence import Audios
-        return QuerySequenceCollection(repo=self, seq_cls=Audios, query=query)
+        return QuerySequenceCollection(repo=self, seq_cls=Audios, query=query, report_mode=report_mode)
 
-    def query_figure_objects(self, query: str = '') -> QuerySequenceCollection:
+    def query_figure_objects(self,
+                             query: str = '',
+                             report_mode: QueryReportMode = QueryReportMode.PROGRESS_BAR) -> QuerySequenceCollection:
         """Get Figures collections satisfying query expression.
 
         Args:
              query (str): query expression.
+             report_mode(:obj:`QueryReportMode`, optional): indicates report mode
+                (0: DISABLED, 1: PROGRESS BAR, 2: PROGRESS TUPLE). QueryReportMode.PROGRESS_BAR if not specified.
         Returns:
             :obj:`SequenceCollection`: Iterable for Figure sequences matching query expression.
         """
         self._prepare_runs_cache()
         from aim.sdk.sequences.figure_sequence import Figures
-        return QuerySequenceCollection(repo=self, seq_cls=Figures, query=query)
+        return QuerySequenceCollection(repo=self, seq_cls=Figures, query=query, report_mode=report_mode)
 
-    def query_distributions(self, query: str = '') -> QuerySequenceCollection:
+    def query_distributions(self,
+                            query: str = '',
+                            report_mode: QueryReportMode = QueryReportMode.PROGRESS_BAR) -> QuerySequenceCollection:
         """Get distribution collections satisfying query expression.
 
         Args:
              query (str): query expression.
+             report_mode(:obj:`QueryReportMode`, optional): indicates report mode
+                (0: DISABLED, 1: PROGRESS BAR, 2: PROGRESS TUPLE). QueryReportMode.PROGRESS_BAR if not specified.
         Returns:
             :obj:`SequenceCollection`: Iterable for distribution sequences matching query expression.
         """
         self._prepare_runs_cache()
         from aim.sdk.sequences.distribution_sequence import Distributions
-        return QuerySequenceCollection(repo=self, seq_cls=Distributions, query=query)
+        return QuerySequenceCollection(repo=self, seq_cls=Distributions, query=query, report_mode=report_mode)
 
-    def query_texts(self, query: str = '') -> QuerySequenceCollection:
+    def query_texts(self,
+                    query: str = '',
+                    report_mode: QueryReportMode = QueryReportMode.PROGRESS_BAR) -> QuerySequenceCollection:
         """Get text collections satisfying query expression.
 
         Args:
              query (str): query expression.
+             report_mode(:obj:`QueryReportMode`, optional): indicates report mode
+                (0: DISABLED, 1: PROGRESS BAR, 2: PROGRESS TUPLE). QueryReportMode.PROGRESS_BAR if not specified.
         Returns:
             :obj:`SequenceCollection`: Iterable for text sequences matching query expression.
         """
         self._prepare_runs_cache()
         from aim.sdk.sequences.text_sequence import Texts
-        return QuerySequenceCollection(repo=self, seq_cls=Texts, query=query)
+        return QuerySequenceCollection(repo=self, seq_cls=Texts, query=query, report_mode=report_mode)
 
     @property
     def run_props_cache_hint(self):
@@ -690,7 +810,7 @@ class Repo:
         # try to acquire a lock on a run container to check if it is still in progress or not
         # in progress runs can't be deleted
         lock_path = os.path.join(self.path, 'meta', 'locks', run_hash)
-        lock = FileLock(str(lock_path), timeout=0)
+        lock = AutoFileLock(lock_path, timeout=0)
         lock.acquire()
 
         with self.structured_db:  # rollback db entity delete if subsequent actions fail.
@@ -719,7 +839,7 @@ class Repo:
         # try to acquire a lock on a run container to check if it is still in progress or not
         # in progress runs can't be copied
         lock_path = os.path.join(self.path, 'meta', 'locks', run_hash)
-        lock = FileLock(str(lock_path), timeout=0)
+        lock = AutoFileLock(lock_path, timeout=0)
         lock.acquire()
         with dest_repo.structured_db:  # rollback destination db entity if subsequent actions fail.
             # copy run structured data
@@ -746,7 +866,7 @@ class Repo:
                 'meta', run_hash, read_only=False, from_union=True
             ).subtree('meta').subtree('chunks').subtree(run_hash)
             dest_meta_run_tree[...] = source_meta_run_tree[...]
-            dest_index = dest_repo._get_index_tree('meta', timeout=0).view(b'')
+            dest_index = dest_repo._get_index_tree('meta', timeout=0).view(())
             dest_meta_run_tree.finalize(index=dest_index)
 
             # copy run series tree
@@ -766,3 +886,11 @@ class Repo:
     @property
     def is_remote_repo(self):
         return self._client is not None
+
+    @contextmanager
+    def atomic_track(self, queue_id):
+        if self.is_remote_repo:
+            self._client.start_instructions_batch()
+        yield
+        if self.is_remote_repo:
+            self._client.flush_instructions_batch(queue_id)
