@@ -7,28 +7,25 @@ import pytz
 import sys
 
 from collections import defaultdict
-from copy import deepcopy
 
+from aim.sdk.base_run import BaseRun
 from aim.sdk.sequence import Sequence
-from aim.storage.object import CustomObject
+from aim.sdk.tracker import RunTracker
+from aim.sdk.reporter import RunStatusReporter
 from aim.sdk.sequence_collection import SingleRunSequenceCollection
 from aim.sdk.utils import (
-    check_types_compatibility,
-    generate_run_hash,
-    get_object_typename,
+    backup_run,
 )
 from aim.ext.utils import (
     get_environment_variables,
     get_installed_packages,
     get_git_info,
 )
-from aim.sdk.num_utils import convert_to_py_number, is_number
 from aim.sdk.types import AimObject
-from aim.sdk.configs import AIM_ENABLE_TRACKING_THREAD, AIM_RUN_INDEXING_TIMEOUT
+from aim.sdk.configs import AIM_RUN_INDEXING_TIMEOUT
 
-from aim.storage.hashing import hash_auto
-from aim.storage.context import Context, SequenceDescriptor
 from aim.storage.treeview import TreeView
+from aim.storage.context import Context
 from aim.storage import treeutils
 
 from aim.ext.resource import ResourceTracker, DEFAULT_SYSTEM_TRACKING_INT
@@ -47,6 +44,7 @@ if TYPE_CHECKING:
     from aim.sdk.sequences.figure_sequence import Figures
     from aim.sdk.sequences.text_sequence import Texts
     from aim.sdk.sequence_collection import SequenceCollection
+    from aim.ext.resource.log import Logs
     from aim.sdk.repo import Repo
 
 logger = logging.getLogger(__name__)
@@ -69,6 +67,9 @@ class RunAutoClean(AutoClean['Run']):
         self.meta_run_tree = instance.meta_run_tree
         self.repo = instance.repo
         self._system_resource_tracker = instance._system_resource_tracker
+        # this reference is needed for system resource tracker finalization
+        self._tracker = instance._tracker
+        self._checkins = instance._checkins
 
     def finalize_run(self):
         """
@@ -77,7 +78,7 @@ class RunAutoClean(AutoClean['Run']):
         self.meta_run_tree['end_time'] = datetime.datetime.now(pytz.utc).timestamp()
         try:
             timeout = os.getenv(AIM_RUN_INDEXING_TIMEOUT, 2 * 60)
-            index = self.repo._get_index_tree('meta', timeout=timeout).view(b'')
+            index = self.repo._get_index_tree('meta', timeout=timeout).view(())
             logger.debug(f'Indexing Run {self.hash}...')
             self.meta_run_tree.finalize(index=index)
         except TimeoutError:
@@ -91,6 +92,11 @@ class RunAutoClean(AutoClean['Run']):
             logger.debug('Stopping resource tracker')
             self._system_resource_tracker.stop()
 
+    def finalize_rpc_queue(self):
+        if self.repo.is_remote_repo:
+            self.repo._client.get_queue(self.hash).stop()
+            self.repo._client.remove_queue(self.hash)
+
     def _close(self) -> None:
         """
         Close the `Run` instance resources and trigger indexing.
@@ -100,6 +106,9 @@ class RunAutoClean(AutoClean['Run']):
             return
         self.finalize_system_tracker()
         self.finalize_run()
+        self.finalize_rpc_queue()
+        if self._checkins is not None:
+            self._checkins.close()
 
 
 # TODO: [AT] generate automatically based on ModelMappedRun
@@ -154,6 +163,20 @@ class StructuredRunMixin:
         """
         return self.props.creation_time
 
+    @property
+    def created_at(self):
+        if self.repo.is_remote_repo:
+            return datetime.datetime.fromtimestamp(self.creation_time, tz=pytz.utc).replace(tzinfo=None)
+        else:
+            return self.props.created_at
+
+    @property
+    def finalized_at(self):
+        if self.end_time:
+            return datetime.datetime.fromtimestamp(self.end_time, tz=pytz.utc).replace(tzinfo=None)
+        else:
+            return None
+
     @property
     def end_time(self):
         """Run finalization time [UTC] as timestamp.
@@ -166,6 +189,31 @@ class StructuredRunMixin:
             # run saved with old version. fallback to sqlite data
             return self.props.end_time
 
+    @property
+    def duration(self):
+        """Run duration in seconds (end_time - creation_time)
+
+            :getter: Returns run duration.
+        """
+        if self.end_time:
+            return self.end_time - self.creation_time
+        else:
+            from aim.web.api.utils import datetime_now
+            return datetime_now().timestamp() - self.creation_time
+
+    @property
+    def active(self):
+        """Check if run is active or not.
+
+            :getter: Returns run's active state.
+            :type: bool
+        """
+
+        if self.end_time:
+            return False
+        else:
+            return True
+
     @property
     def experiment(self):
         """Run experiment.
@@ -205,18 +253,7 @@ class StructuredRunMixin:
         return self.props.remove_tag(tag_name)
 
 
-class SequenceInfo:
-    def __init__(self):
-        self.initialized = False
-        self.count = None
-        self.sequence_dtype = None
-        self.val_view = None
-        self.epoch_view = None
-        self.time_view = None
-        self.record_max_length = None
-
-
-class Run(StructuredRunMixin):
+class Run(BaseRun, StructuredRunMixin):
     """Run object used for tracking metrics.
 
     Provides method :obj:`track` to track value and object series for multiple names and contexts.
@@ -237,49 +274,52 @@ class Run(StructuredRunMixin):
             git info, environment variables, etc.
     """
 
-    _idx_to_ctx: Dict[int, Context] = dict()
-    _track_warning_shown = False
-
-    track_in_thread = os.getenv(AIM_ENABLE_TRACKING_THREAD, False)
+    _metric_version_warning_shown = False
 
     def __init__(self, run_hash: Optional[str] = None, *,
                  repo: Optional[Union[str, 'Repo']] = None,
                  read_only: bool = False,
                  experiment: Optional[str] = None,
                  system_tracking_interval: Optional[Union[int, float]] = DEFAULT_SYSTEM_TRACKING_INT,
-                 log_system_params: Optional[bool] = False):
+                 log_system_params: Optional[bool] = False,
+                 capture_terminal_logs: Optional[bool] = True):
         self._resources: Optional[RunAutoClean] = None
-        run_hash = run_hash or generate_run_hash()
-        self.hash = run_hash
-
-        self._finalized = False
-
-        self.repo: Repo = None
-        self._set_repo(repo)
-
-        self.read_only = read_only
-        if not read_only:
-            logger.debug(f'Opening Run {self.hash} in write mode')
-
-        self._hash = None
-        self._props = None
-
-        self.contexts: Dict[Context, int] = dict()
-        self.sequence_info: Dict[SequenceDescriptor.Selector, SequenceInfo] = defaultdict(SequenceInfo)
-
-        self.meta_tree: TreeView = self.repo.request_tree(
-            'meta', self.hash, read_only=read_only, from_union=True
-        ).subtree('meta')
-        self.meta_run_tree: TreeView = self.meta_tree.subtree('chunks').subtree(self.hash)
+        super().__init__(run_hash, repo=repo, read_only=read_only)
 
         self.meta_attrs_tree: TreeView = self.meta_tree.subtree('attrs')
         self.meta_run_attrs_tree: TreeView = self.meta_run_tree.subtree('attrs')
 
-        self.series_run_tree: TreeView = self.repo.request_tree(
-            'seqs', self.hash, read_only=read_only
-        ).subtree('seqs').subtree('chunks').subtree(self.hash)
+        if not read_only:
+            logger.debug(f'Opening Run {self.hash} in write mode')
+
+            if self.check_metrics_version():
+                if self.repo.is_remote_repo:
+                    logger.warning(f'Cannot track Run with remote repo {self.repo.path}. Please upgrade repo first '
+                                   f'with the following command:')
+                    logger.warning(f'aim storage --repo {self.repo.path} upgrade 3.11+ \'*\'')
+                    raise RuntimeError
+                else:
+                    logger.warning(f'Detected sub-optimal format metrics for Run {self.hash}. Upgrading...')
+                    backup_path = backup_run(self)
+                    try:
+                        self.update_metrics()
+                        logger.warning(f'Successfully converted Run {self.hash}')
+                        logger.warning(f'Run backup can be found at {backup_path}. '
+                                       f'In case of any issues the following command can be used to restore data: '
+                                       f'`aim storage --repo {self.repo.root_path} restore {self.hash}`')
+                    except Exception as e:
+                        logger.error(f'Failed to convert metrics. {e}')
+                        logger.warning(f'Run backup can be found at {backup_path}. '
+                                       f'To restore data please run the following command: '
+                                       f'`aim storage --repo {self.repo.root_path} restore {self.hash}`')
+                        raise
+
+        self._props = None
+        self._checkins = None
 
         if not read_only:
+            if not self.repo.is_remote_repo:
+                self._checkins = RunStatusReporter(self)
             if log_system_params:
                 system_params = {
                     'packages': get_installed_packages(),
@@ -291,7 +331,7 @@ class Run(StructuredRunMixin):
                 self.__setitem__("__system_params", system_params)
 
             try:
-                self.meta_run_attrs_tree.first()
+                self.meta_run_attrs_tree.first_key()
             except (KeyError, StopIteration):
                 # no run params are set. use empty dict
                 self[...] = {}
@@ -300,24 +340,18 @@ class Run(StructuredRunMixin):
         if experiment:
             self.experiment = experiment
 
-        self._prepare_sequence_info(read_only)
+        self._tracker = RunTracker(self)
 
         self._system_resource_tracker: ResourceTracker = None
-        self._prepare_resource_tracker(system_tracking_interval)
+        self._prepare_resource_tracker(system_tracking_interval, capture_terminal_logs)
 
         self._resources = RunAutoClean(self)
 
-    def __repr__(self) -> str:
-        return f'<Run#{hash(self)} name={self.hash} repo={self.repo}>'
+    def __hash__(self) -> int:
+        return super().__hash__()
 
     def idx_to_ctx(self, idx: int) -> Context:
-        ctx = Run._idx_to_ctx.get(idx)
-        if ctx is not None:
-            return ctx
-        ctx = Context(self.meta_tree['contexts', idx])
-        Run._idx_to_ctx[idx] = ctx
-        self.contexts[ctx] = idx
-        return ctx
+        return self._tracker.idx_to_ctx(idx)
 
     def __setitem__(self, key: str, val: Any):
         """Set Run top-level meta-parameter.
@@ -362,16 +396,22 @@ class Run(StructuredRunMixin):
     def _collect(self, key, strict: bool = True, resolve_objects: bool = False):
         return self.meta_run_attrs_tree.collect(key, strict=strict, resolve_objects=resolve_objects)
 
-    def _prepare_resource_tracker(self, tracking_interval: Union[int, float] = None):
-        if not self.read_only and isinstance(tracking_interval, (int, float)) and tracking_interval > 0:
-            try:
-                self._system_resource_tracker = ResourceTracker(self.track, tracking_interval)
-            except ValueError:
-                print('To track system resource usage '
-                      'please set `system_tracking_interval` greater than 0 '
-                      'and less than 1 day')
-            else:
-                self._system_resource_tracker.start()
+    def _prepare_resource_tracker(
+            self,
+            tracking_interval: Union[int, float] = None,
+            capture_terminal_logs: bool = True
+    ):
+        if self.read_only:
+            return
+
+        if ResourceTracker.check_interval(tracking_interval) or capture_terminal_logs:
+            current_logs = self.get_terminal_logs()
+            log_offset = current_logs.last_step() + 1 if current_logs else 0
+            self._system_resource_tracker = ResourceTracker(self._tracker,
+                                                            tracking_interval,
+                                                            capture_terminal_logs,
+                                                            log_offset)
+            self._system_resource_tracker.start()
 
     def __delitem__(self, key: str):
         """Remove key from run meta-params.
@@ -384,7 +424,7 @@ class Run(StructuredRunMixin):
     def track(
         self,
         value,
-        name: str,
+        name: str = None,
         step: int = None,
         epoch: int = None,
         *,
@@ -403,57 +443,7 @@ class Run(StructuredRunMixin):
         Appended values should be of the same type, in other words, sequence is a homogeneous collection.
         """
 
-        # since worker might be lagging behind, we want to log the timestamp of run.track() call,
-        # not the actual implementation execution time.
-        track_time = datetime.datetime.now(pytz.utc).timestamp()
-        if self.track_in_thread:
-            val = deepcopy(value)
-            track_rate_warning = self.repo.tracking_queue.register_task(
-                self._track_impl, val, track_time, name, step, epoch, context=context)
-            if track_rate_warning:
-                self.track_rate_warn()
-        else:
-            self._track_impl(value, track_time, name, step, epoch, context=context)
-
-    def _track_impl(
-        self,
-        value,
-        track_time: float,
-        name: str,
-        step: int = None,
-        epoch: int = None,
-        *,
-        context: AimObject = None,
-    ):
-        if context is None:
-            context = {}
-
-        if is_number(value):
-            val = convert_to_py_number(value)
-        elif isinstance(value, (CustomObject, list, tuple)):
-            val = value
-        else:
-            raise ValueError(f'Input metric of type {type(value)} is neither python number nor AimObject')
-
-        ctx = Context(context)
-        if ctx not in self.contexts:
-            self.meta_tree['contexts', ctx.idx] = context
-            self.meta_run_tree['contexts', ctx.idx] = context
-            self.contexts[ctx] = ctx.idx
-            self._idx_to_ctx[ctx.idx] = ctx
-
-        seq_info = self._get_or_create_sequence_info(ctx, name)
-        step = step if step is not None else seq_info.count
-        self._update_sequence_info(seq_info, ctx, val, name, step)
-
-        self.meta_run_tree['traces', ctx.idx, name, 'last'] = val
-        self.meta_run_tree['traces', ctx.idx, name, 'last_step'] = step
-
-        # TODO perform assignments in an atomic way
-        seq_info.val_view[step] = val
-        seq_info.epoch_view[step] = epoch
-        seq_info.time_view[step] = track_time
-        seq_info.count = seq_info.count + 1
+        self._tracker(value, name, step, epoch, context=context)
 
     @property
     def props(self):
@@ -505,7 +495,9 @@ class Run(StructuredRunMixin):
             >>> for metric in run.metrics():
             >>>     metric.values.sparse_numpy()
         """
-        return SingleRunSequenceCollection(self)
+        from aim.sdk.sequences.metric import Metric
+        self.repo._prepare_runs_cache()
+        return SingleRunSequenceCollection(self, seq_cls=Metric)
 
     def __eq__(self, other: 'Run') -> bool:
         return self.hash == other.hash and self.repo == other.repo
@@ -524,6 +516,13 @@ class Run(StructuredRunMixin):
         Returns:
             :obj:`Metric` object if exists, `None` otherwise.
         """
+        if self.read_only and not Run._metric_version_warning_shown:
+            if self.check_metrics_version():
+                logger.warning(f'Detected sub-optimal format metrics for Run {self.hash}. Consider upgrading repo '
+                               f'to improve queries performance:')
+                logger.warning(f'aim storage --repo {self.repo.path} upgrade 3.11+ \'*\'')
+                Run._metric_version_warning_shown = True
+
         return self._get_sequence('metric', name, context)
 
     def get_image_sequence(
@@ -590,6 +589,14 @@ class Run(StructuredRunMixin):
         """
         return self._get_sequence('distributions', name, context)
 
+    def get_terminal_logs(self) -> Optional['Logs']:
+        """Retrieve duplicated terminal logs for a run
+
+                Returns:
+                    :obj:`Logs` object if exists, `None` otherwise.
+                """
+        return self._get_sequence('logs', 'logs', Context({}))
+
     def get_text_sequence(
             self,
             name: str,
@@ -633,76 +640,11 @@ class Run(StructuredRunMixin):
         sequence = seq_cls(sequence_name, context, self)
         return sequence if bool(sequence) else None
 
-    def _prepare_sequence_info(self, read_only):
-        if read_only:
-            return
-
-        for ctx_id, traces in self.meta_run_tree.get('traces', {}).items():
-            for name in traces:
-                try:
-                    self._read_sequence_info(self.idx_to_ctx(ctx_id), name)
-                except KeyError:
-                    pass
-
-    def _read_sequence_info(self, ctx, name):
-        sequence_selector = SequenceDescriptor(name, ctx).selector
-        seq_info = self.sequence_info[sequence_selector]
-
-        assert not seq_info.initialized
-        seq_info.val_view = self.series_run_tree.subtree(sequence_selector).array('val').allocate()
-        seq_info.epoch_view = self.series_run_tree.subtree(sequence_selector).array('epoch').allocate()
-        seq_info.time_view = self.series_run_tree.subtree(sequence_selector).array('time').allocate()
-        seq_info.count = len(seq_info.val_view)
-        seq_info.sequence_dtype = self.meta_run_tree.get(('traces', ctx.idx, name, 'dtype'), None)
-        seq_info.record_max_length = self.meta_run_tree.get(('traces', ctx.idx, name, 'record_max_length'), 0)
-        seq_info.initialized = True
-
-    def _get_or_create_sequence_info(self, ctx, name):
-        # this method is used in the `run.track()`, so please use only write-only instructions
-        sequence_selector = SequenceDescriptor(name, ctx).selector
-        seq_info = self.sequence_info[sequence_selector]
-
-        if seq_info.initialized:
-            return seq_info
-
-        # the subtree().array().allocate() method is write-only
-        seq_info.val_view = self.series_run_tree.subtree(sequence_selector).array('val').allocate()
-        seq_info.epoch_view = self.series_run_tree.subtree(sequence_selector).array('epoch').allocate()
-        seq_info.time_view = self.series_run_tree.subtree(sequence_selector).array('time').allocate()
-        seq_info.count = 0
-        seq_info.sequence_dtype = None
-        seq_info.record_max_length = 0
-        seq_info.initialized = True
-        return seq_info
-
-    def _update_sequence_info(self, seq_info, ctx, val, name, step):
-        # this method is used in the `run.track()`, so please use only write-only instructions
-        dtype = get_object_typename(val)
-
-        if seq_info.sequence_dtype is not None:
-            def update_trace_dtype(new_dtype):
-                self.meta_tree['traces_types', new_dtype, ctx.idx, name] = 1
-                self.meta_run_tree['traces', ctx.idx, name, 'dtype'] = new_dtype
-                seq_info.sequence_dtype = new_dtype
-
-            compatible = check_types_compatibility(dtype, seq_info.sequence_dtype, update_trace_dtype)
-            if not compatible:
-                raise ValueError(f'Cannot log value \'{val}\' on sequence \'{name}\'. Incompatible data types.')
-
-        if seq_info.count == 0:
-            self.meta_tree['traces_types', dtype, ctx.idx, name] = 1
-            self.meta_run_tree['traces', ctx.idx, name, 'dtype'] = dtype
-            self.meta_run_tree['traces', ctx.idx, name, 'first_step'] = step
-            seq_info.sequence_dtype = dtype
-
-        if isinstance(val, (tuple, list)):
-            record_max_length = max(seq_info.record_max_length, len(val))
-            self.meta_run_tree['traces', ctx.idx, name, 'record_max_length'] = record_max_length
-            seq_info.record_max_length = record_max_length
-
-        return seq_info
-
-    def collect_sequence_info(self, sequence_types: Tuple[str, ...], skip_last_value=False) -> Dict[str, list]:
+    def collect_sequence_info(
+            self,
+            sequence_types: Union[str, Tuple[str, ...]],
+            skip_last_value=False
+    ) -> Dict[str, list]:
         """Retrieve Run's all sequences general overview.
 
         Args:
@@ -745,45 +687,29 @@ class Run(StructuredRunMixin):
                         traces_overview[seq_type].append(trace_data)
         return traces_overview
 
-    def _calc_hash(self) -> int:
-        # TODO maybe take read_only flag into account?
-        return hash_auto((self.hash, hash(self.repo)))
-
-    def _set_repo(self, repo):
-        if repo is None:
-            from aim.sdk.repo import Repo
-            repo = Repo.default_repo_path()
-        if isinstance(repo, str):
-            from aim.sdk.repo import Repo, RepoStatus
-            repo_status = Repo.check_repo_status(repo)
-            if repo_status == RepoStatus.UPDATE_REQUIRED:
-                logger.error(f'Trying to start Run on repository {repo}, which is out of date. '
-                             f'Please upgrade repository with the following command: '
-                             f'`aim upgrade --repo {repo} 2to3`.')
-                raise RuntimeError()
-            elif repo_status == RepoStatus.MISSING:
-                repo = Repo.from_path(repo, init=True)
-            else:
-                repo = Repo.from_path(repo)
-        self.repo = repo
-
-    def __hash__(self) -> int:
-        if self._hash is None:
-            self._hash = self._calc_hash()
-        return self._hash
+    def _cleanup_trees(self):
+        del self.meta_run_attrs_tree
+        del self.meta_attrs_tree
+        del self.meta_run_tree
+        del self.meta_tree
+        del self._series_run_trees
+        self.meta_run_attrs_tree = None
+        self.meta_run_tree = None
+        self.meta_attrs_tree = None
+        self.meta_tree = None
+        self._series_run_trees = None
 
     def close(self):
         if self._resources is None:
             return
         self._resources.close()
-
-    @classmethod
-    def track_rate_warn(cls):
-        if not cls._track_warning_shown:
-            # TODO [AT] add link to FAQ section in docs.
-            logger.warning('Tracking task queue is almost full which might cause performance degradation. '
-                           'Consider tracking at lower pace.')
-            cls._track_warning_shown = True
+        self._tracker.sequence_infos.clear()
+        # de-reference trees and other resources
+        del self._resources
+        del self._props
+        self._resources = None
+        self._props = None
+        self._cleanup_trees()
 
     def finalize(self):
         if self._resources is None:
@@ -814,7 +740,8 @@ class Run(StructuredRunMixin):
             data['description'] = self.props.description
             data['archived'] = self.props.archived
             data['creation_time'] = self.props.creation_time
-            data['end_time'] = self.props.end_time
+            data['end_time'] = self.end_time
+            data['active'] = self.active
             data['experiment'] = self.props.experiment
             data['tags'] = json.dumps(self.props.tags)
 
@@ -840,3 +767,39 @@ class Run(StructuredRunMixin):
         import pandas as pd
         df = pd.DataFrame(data, index=[0])
         return df
+
+    def report_progress(
+        self,
+        *,
+        expect_next_in: int = 0,
+        block: bool = False,
+    ) -> None:
+        """
+        Report progress for the run. Report the expected time for the next progress report.
+
+        If no progress reports are received by the expiry date (plus the grace period), the
+        run is considered as failed.
+
+        Args:
+            expect_next_in: (:obj:`int`, optional): The number of seconds to wait before the next progress report.
+            block: (:obj:`bool`, optional): If true, block the thread until the report is written to filesystem.
+        """
+        if self._checkins is None:
+            raise ValueError('Progress reports are not enabled for this run')
+        self._checkins._check_in(expect_next_in=expect_next_in, block=block)
+
+    def report_successful_finish(
+        self,
+        *,
+        block: bool = True,
+    ) -> None:
+        """
+        Report successful finish of the run. If the run is not marked as successfully finished,
+        it can potentially be considered as failed.
+
+        Args:
+            block: (:obj:`bool`, optional): If true, block the thread until the report is written to filesystem.
+        """
+        if self._checkins is None:
+            raise ValueError('Progress reports are not enabled for this run')
+        self._checkins._report_successful_finish(block=block)
